﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FsrmLib;
using System.Xml;
using System.Runtime.InteropServices;
using System.IO;

namespace AvailTek.FCI.FCI2XML
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length < 2)
            {
                Console.WriteLine("FCI2XML <Directory> <filename> [Recurse: Yes|No]");
                return;
            }

            string paramDirectory = args[0];
            string paramFileName = args[1];
            bool recurse = false;
            if (args.Length >= 3) bool.TryParse(args[2], out recurse);

            DirectoryInfo di = new DirectoryInfo(paramDirectory);
            if (!di.Exists)
            {
                Console.WriteLine("Source Directory does not exist or is unreachable");
                return;
            }

            FsrmClassificationManager FCIClassMgr = null;
            try
            {
                FCIClassMgr = new FsrmClassificationManager();
                ProcessDirectory(di, FCIClassMgr, paramFileName, recurse);
            }
            catch(Exception e)
            {
                Console.WriteLine("Exception {0} encountered", e.ToString());
            }
            finally
            {
                if (FCIClassMgr != null) { Marshal.FinalReleaseComObject(FCIClassMgr); }
            }
        }

        static void ProcessDirectory(DirectoryInfo di, FsrmClassificationManager FCIClassMgr, string xmlFileName, bool recurse)
        {
            // Process Subdirectories
            if (recurse)
            {
                foreach (DirectoryInfo diSub in di.GetDirectories())
                {
                    ProcessDirectory(diSub, FCIClassMgr, xmlFileName, recurse);
                }
            }

            // Process files
            XmlDocument xmlDoc = new XmlDocument();
            XmlElement eleFiles = xmlDoc.CreateElement("Files");
            xmlDoc.AppendChild(eleFiles);

            int processedFiles = 0;
            foreach (FileInfo file in di.GetFiles())
            {
                if (string.Compare(file.Name, xmlFileName, true) != 0)
                {
                    ProcessFile(file, FCIClassMgr, eleFiles);
                    processedFiles++;
                }
            }

            if (processedFiles > 1)
            {
                string metadataFullPath = di.FullName + '\\' + xmlFileName;
                File.Delete(metadataFullPath);
                xmlDoc.Save(metadataFullPath);
            }
        }

        static void ProcessFile(FileInfo file, FsrmClassificationManager FCIClassMgr, XmlElement eleFiles)
        {
            XmlElement eleFile = eleFiles.OwnerDocument.CreateElement("File");
            eleFiles.AppendChild(eleFile);

            XmlAttribute attName = eleFiles.OwnerDocument.CreateAttribute("Name");
            attName.Value = file.Name;
            eleFile.Attributes.Append(attName);

            XmlElement eleProperties = eleFiles.OwnerDocument.CreateElement("Properties");
            eleFile.AppendChild(eleProperties);

            IFsrmCollection properties = FCIClassMgr.EnumFileProperties(file.FullName);
            foreach (IFsrmProperty prop in properties)
            {
                XmlElement eleProp = eleFiles.OwnerDocument.CreateElement("Property");
                eleProperties.AppendChild(eleProp);

                try
                {
                    XmlAttribute attPropName = eleFiles.OwnerDocument.CreateAttribute("Name");
                    eleProp.Attributes.Append(attPropName);
                    attPropName.Value = prop.Name;

                    IFsrmPropertyDefinition propDef = GetPropertyDefinition(prop.Name, FCIClassMgr);
                    XmlAttribute attType = eleFiles.OwnerDocument.CreateAttribute("Type");
                    if (propDef == null) attType.Value = "Custom";
                    else attType.Value = GetFsrmTypeText(propDef.Type);
                    eleProp.Attributes.Append(attType);
                    eleProp.InnerText = prop.Value;
                }
                catch (Exception e)
                {
                    System.Diagnostics.Trace.WriteLine(string.Format("Exception {0} on file {1} property {2}", e.ToString(), file.FullName, prop.Name));
                }
            }
        }

        static string GetFsrmTypeText(_FsrmPropertyDefinitionType type)
        {
            switch (type)
            {
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_Bool: return "Boolean";
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_Date: return "Date";
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_Int: return "Integer";
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_MultiChoiceList: return "Multi Choice List";
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_MultiString: return "Multi String";
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_OrderedList: return "Ordered List";
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_String: return "String";
                case _FsrmPropertyDefinitionType.FsrmPropertyDefinitionType_Unknown: return "Unknown";
            }

            return "Unrecognized";
        }

        static Dictionary<string, IFsrmPropertyDefinition> PropertyDefinitionCache = new Dictionary<string, IFsrmPropertyDefinition>();
        static IFsrmPropertyDefinition GetPropertyDefinition(string name, FsrmClassificationManager FCIClassMgr)
        {
            if (PropertyDefinitionCache.ContainsKey(name)) return PropertyDefinitionCache[name];

            try
            {
                IFsrmPropertyDefinition propDef = FCIClassMgr.GetPropertyDefinition(name);

                if (propDef != null)
                {
                    PropertyDefinitionCache.Add(name, propDef);
                    return propDef;
                }
            }
            catch (Exception e)
            {
                System.Diagnostics.Trace.WriteLine(string.Format("Exception {0} fetching property {1}", e.ToString(), name));
                PropertyDefinitionCache.Add(name, null);
            }

            return null;
        }
    }
}
